#!/usr/bin/osascript
#
use AppleScript version "2.4" -- Yosemite (10.10) or later
use scripting additions

--
-- Remove all commas from String
--
on rmCommas(myString)
	set TID to AppleScript's text item delimiters
	set AppleScript's text item delimiters to ","
	set pieces to text items of myString -- break string apart at commas
	set AppleScript's text item delimiters to "" -- or whatever replaces the comma
	set myString to pieces as text -- put string back together using whatever
	set AppleScript's text item delimiters to TID
	return myString
end rmCommas
--
-- Read Decibel Player Playlist
--
on readPlaylistDecibel()
	tell application "Decibel"
		launch
		set cRunning to {}
		set cTitle to {}
		set cArtist to {}
		set cGenre to {}
		set playingIdTag to {}
		set playingStatus to false
		set readSuccess to false
		--
		try
			if (not (exists nowPlaying)) then -- Player does not contain usefull information yet
				return {cRunning, cTitle, cArtist, cGenre, playingIdTag, playingStatus, readSuccess}
			end if
			set playingIdTag to id of nowPlaying & artist of nowPlaying & title of nowPlaying & genre of nowPlaying
			--
			-- Get index of track currently playing or stopped
			--
			repeat with i from 1 to the count of tracks
				set currentIdTag to id of track i & artist of track i & title of track i & genre of track i
				if (currentIdTag = playingIdTag) then
					exit repeat
				end if
				set i to i + 1
			end repeat
			--
			-- Get up to 7 titles of the playlist
			--
			if (i < 4) and ((count of tracks) > 7) then
				-- First 3 track of playlist and there are more than 7 tracks in playlist
				set startTrack to 1
				set endTrack to 7
			else if ((((count of tracks) - i) < 3) and (count of tracks) > 7) then
				-- Last 3 tracks of playlist and there are more than 7 tracks in playlist
				set startTrack to (count of tracks) - 6
				set endTrack to (count of tracks)
			else if ((count of tracks) < 7) then
				-- Less than 7 tracks in playlist
				set startTrack to 1
				set endTrack to (count of tracks)
			else
				-- Three tracks before current track, three tracks after current track
				set startTrack to i - 3
				set endTrack to i + 3
			end if
			repeat with i from startTrack to endTrack
				if ((title of track i is missing value) or (title of track i is "")) then
					set end of cTitle to "Unbekannter Titel"
				else
					set end of cTitle to title of track i
				end if
				if ((artist of track i is missing value) or (artist of track i is "")) then
					set end of cArtist to "Unbekanntes Orchester"
				else
					set end of cArtist to artist of track i
				end if
				if ((genre of track i is missing value) or (genre of track i is "")) then
					set end of cGenre to "Cortina"
				else
					set end of cGenre to genre of track i
				end if
				set currentIdTag to id of track i & artist of track i & title of track i & genre of track i
				if (playing and (currentIdTag = playingIdTag)) then
					set end of cRunning to "Playing"
					set playingStatus to true
				else if (currentIdTag = playingIdTag) then
					set end of cRunning to "Stopped"
				else
					set end of cRunning to "False"
				end if
				set i to i + 1
			end repeat
			set readSuccess to true
			return {cRunning, cTitle, cArtist, cGenre, playingIdTag, playingStatus, readSuccess}
		on error
			return {cRunning, cTitle, cArtist, cGenre, playingIdTag, playingStatus, readSuccess}
		end try
	end tell
end readPlaylistDecibel

--
-- Read Music Player Playlist
--
on readPlaylistMusic()
	tell application "iTunes" -- change to "Music" if system version of (system info) >= "10.15"
		launch
		set cRunning to {}
		set cTitle to {}
		set cArtist to {}
		set cGenre to {}
		set playingIdTag to {}
		set playingStatus to false
		set readSuccess to false
		--
		try
			if (not (exists (name of current playlist))) then -- Player does not contain usefull information yet
				return {cRunning, cTitle, cArtist, cGenre, playingIdTag, playingStatus, readSuccess}
			end if
			set playingIdTag to (id of current track as string) & artist of current track & name of current track & genre of current track
			--
			-- Get index of track currently playing or stopped
			--
			repeat with i from 1 to the count of tracks in current playlist
				set currentIdTag to (id of track i of current playlist as string) & artist of track i of current playlist & name of track i of current playlist & genre of track i of current playlist
				if (currentIdTag = playingIdTag) then
					exit repeat
				end if
				set i to i + 1
			end repeat
			--
			-- Get up to 7 titles of the playlist
			--
			if (i < 4) and ((count of tracks in current playlist) > 7) then
				-- First 3 track of playlist and there are more than 7 tracks in playlist
				set startTrack to 1
				set endTrack to 7
			else if ((((count of tracks in current playlist) - i) < 3) and (count of tracks in current playlist) > 7) then
				-- Last 3 tracks of playlist and there are more than 7 tracks in playlist
				set startTrack to (count of tracks in current playlist) - 6
				set endTrack to (count of tracks in current playlist)
			else if ((count of tracks in current playlist) < 7) then
				-- Less than 7 tracks in playlist
				set startTrack to 1
				set endTrack to (count of tracks in current playlist)
			else
				-- Three tracks before current track, three tracks after current track
				set startTrack to i - 3
				set endTrack to i + 3
			end if
			repeat with i from startTrack to endTrack
				if ((name of track i of current playlist is missing value) or (name of track i of current playlist is "")) then
					set end of cTitle to "Unbekannter Titel"
				else
					set end of cTitle to name of track i of current playlist
				end if
				if ((artist of track i of current playlist is missing value) or (artist of track i of current playlist is "")) then
					set end of cArtist to "Unbekanntes Orchester"
				else
					set end of cArtist to artist of track i of current playlist
				end if
				if ((genre of track i of current playlist is missing value) or (genre of track i of current playlist is "")) then
					set end of cGenre to "Cortina"
				else
					set end of cGenre to genre of track i of current playlist
				end if
				set currentIdTag to (id of track i of current playlist as string) & artist of track i of current playlist & name of track i of current playlist & genre of track i of current playlist
				if ((player state is playing) and (currentIdTag = playingIdTag)) then
					set end of cRunning to "Playing"
					set playingStatus to true
				else if (currentIdTag = playingIdTag) then
					set end of cRunning to "Stopped"
				else
					set end of cRunning to "False"
				end if
				set i to i + 1
			end repeat
			set readSuccess to true
			return {cRunning, cTitle, cArtist, cGenre, playingIdTag, playingStatus, readSuccess}
		on error
			return {cRunning, cTitle, cArtist, cGenre, playingIdTag, playingStatus, readSuccess}
		end try
	end tell
end readPlaylistMusic

--
-- Read Swinsian Player Playlist
--
on readPlaylistSwinsian()
	tell application "Swinsian"
		launch
		set cRunning to {}
		set cTitle to {}
		set cArtist to {}
		set cGenre to {}
		set playingIdTag to {}
		set playingStatus to false
		set readSuccess to false
		--
		set selected to tracks of playback queue
		try
			if selected is not {} then
				set lastRunning to (id of current track as string) & artist of current track & name of current track & genre of current track
				if ((count of selected's items) > 7) then
					set numTracks to 7
				else
					set numTracks to count of selected's items
				end if
				repeat with i from 1 to numTracks
					if ((title of item i of selected is missing value) or (title of item i of selected is "")) then
						set end of cTitle to "Unbekannter Titel"
					else
						set end of cTitle to title of item i of selected
					end if
					if ((artist of item i of selected is missing value) or (artist of item i of selected is "")) then
						set end of cArtist to "Unbekanntes Orchester"
					else
						set end of cArtist to artist of item i of selected
					end if
					if ((genre of item i of selected is missing value) or (genre of item i of selected is "")) then
						set end of cGenre to "Cortina"
					else
						set end of cGenre to genre of item i of selected
					end if
					set currentIdTag to (id of item i of selected as string) & artist of item i of selected & name of item i of selected & genre of item i of selected
					if ((player state is playing) and (currentIdTag = lastRunning)) then
						set end of cRunning to "Playing"
						set playingStatus to true
					else if (currentIdTag = lastRunning) then
						set end of cRunning to "Stopped"
					else
						set end of cRunning to "False"
					end if
				end repeat
			end if
			set readSuccess to true
			return {cRunning, cTitle, cArtist, cGenre, lastRunning, playingStatus, readSuccess}
		on error
			return {cRunning, cTitle, cArtist, cGenre, lastRunning, playingStatus, readSuccess}
		end try
	end tell
end readPlaylistSwinsian

on run argv
	--
	-- usage: /usr/bin/osascript runAsScript7Tracks.scpt [Decibel|Music|Swinsian]
	--
	-- Default musicPlayer if no argument is given
	--
	set musicPlayer to "Decibel"
	if (count of argv) > 0 then
		set musicPlayer to item 1 of argv
	end if
	--
	-- Read playlist from "Music", "Swinsian" or "Decibel" musicPlayer. On error, numTitlesInPlaylist (count of cRunning) is 0
	--
	if musicPlayer = "Music" then
		set {cRunning, cTitle, cArtist, cGenre, playingIdTag, playingStatus, readSuccess} to readPlaylistMusic()
	else if musicPlayer = "Swinsian" then
		set {cRunning, cTitle, cArtist, cGenre, playingIdTag, playingStatus, readSuccess} to readPlaylistSwinsian()
	else if musicPlayer = "Decibel" then
		set {cRunning, cTitle, cArtist, cGenre, playingIdTag, playingStatus, readSuccess} to readPlaylistDecibel()
	else
		-- Default musicPlayer if argument is an unsupported musicPlayer
		set musicPlayer to "Decibel"
		set {cRunning, cTitle, cArtist, cGenre, playingIdTag, playingStatus, readSuccess} to readPlaylistDecibel()
	end if
	--
	-- Delete any commas from arrays
	--
	set numTitlesInPlaylist to count of cRunning
	if numTitlesInPlaylist > 0 then
		repeat with i from 1 to numTitlesInPlaylist
			set item i of cTitle to rmCommas(item i of cTitle)
			set item i of cArtist to rmCommas(item i of cArtist)
			set item i of cGenre to rmCommas(item i of cGenre)
		end repeat
		set playingIdTag to rmCommas(playingIdTag)
	end if
	return {numTitlesInPlaylist, cRunning, cTitle, cArtist, cGenre, playingIdTag, playingStatus, readSuccess}
end run

