unit displayplaylist_u1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls, RegExpr,
  Process, dateutils, Crt, lclintf, Menus, inifiles, Unix, StrUtils, FileUtil;

type

  { TDisplayPlaylistGUI }

  TDisplayPlaylistGUI = class(TForm)
    StartBrowser: TLabel;
    LabelCurrentStatus: TLabel;
    LabelCurrentSong: TLabel;
    LabelAppleScript: TLabel;
    LabelStatus: TLabel;
    Quit: TButton;
    ComboBoxMusicPlayer: TComboBox;
    ComboBoxAdvertising: TComboBox;
    ImageElDuende: TImage;
    LabelWerbung: TLabel;
    LabelMusicPlayer: TLabel;
    Copyright: TLabel;
    procedure ComboBoxAdvertisingSelect(Sender: TObject);
    procedure ComboBoxmusicPlayerSelect(Sender: TObject);
    procedure CopyrightClick(Sender: TObject);
    procedure CopyrightMouseEnter(Sender: TObject);
    procedure CopyrightMouseLeave(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ImageElDuendeClick(Sender: TObject);
    procedure ImageElDuendeMouseEnter(Sender: TObject);
    procedure QuitClick(Sender: TObject);
    procedure StartBrowserClick(Sender: TObject);
    procedure StartBrowserMouseEnter(Sender: TObject);
    procedure StartBrowserMouseLeave(Sender: TObject);
  private

  public

  end;

var
  DisplayPlaylistGUI: TDisplayPlaylistGUI;

implementation

var
  // .DisplayPlaylist init file variables
  macUser, macSitesDir, macHostName, browserUrl: string;
  runAsScriptProg, advertising, musicPlayer, changeToCortina: string;
  // LocalMac files
  displayPlaylistHtml, displayPlaylistHeader : string;
  // variables used in playlist
  cRunning, cTitle, cArtist, cGenre: TStringArray;
  playingIdTag, currentSong, currentStatus: string;
  playingStatus, readSuccess: boolean;
  numTitlesInPlaylist: longint;
  // The command line for /usr/bin/osascript
  Aprocess : Tprocess;
  AstringList : TstringList;
  Status : string;
  runAsScript: string;

{$R *.lfm}

{ TDisplayPlaylistGUI }

procedure MyDelay(milliSecondsDelay: int64);
// Insert a delay of given number of milliSeconds. During the delay
// the GUI is checked to recognize changes in the GUI.
var
  stopTime: TDateTime;
begin
  stopTime := IncMilliSecond(Now, milliSecondsDelay);
  while (Now < stopTime) and (not Application.Terminated) do
    Application.ProcessMessages;
end;

procedure execOsaScript(s: string);
// Call AppleScript from Pascal and return result in variable 'Status'.
// execOsaScript('Decibel'|'Music'|'Swinsian'|'OtherMusicPlayer')
begin
  Aprocess := Tprocess.Create(nil);
  Aprocess.Executable := '/usr/bin/osascript';
  Aprocess.Parameters.Add(runAsScript);
  if (s <> '') then
     Aprocess.Parameters.Add(s);
  Aprocess.Options := Aprocess.Options + [poWaitOnExit, poUsePipes];
  Aprocess.Execute;
  AstringList := TStringList.Create;
  AstringList.LoadFromStream(Aprocess.Output);
  Status := AstringList.Text;
  AStringList.Free;
  Aprocess.Free;
end;

function seperateNumWithSpace(s: string): string;
// Seperate numbers with a space. More than one number is allowed.
// Remove multiple spaces in string and format with AnsiProperCase.
// As an example change string from 'Biagi45Ortiz' to 'Biagi 45 Ortis'
// This is done for better readability of the string.
var
  r : string;
  i : longint;
  ascii : boolean;
begin
  // Remove spaces from beginning and end of string
  // Change multiple white spaces to single space
  //
  // s := trim(ReplaceRegExpr('\s+',s,' ',False));  // White space chars
  s := trim(DelSpace1(s));                    // Space chars
  r := '';
  ascii:= true;
  for i:=1 to length(s) do
  begin
    if (s[i] in ['0'..'9']) and (ascii = true) then
    begin
      ascii:=false;
      r:= r + ' ';
    end;
    if not (s[i] in ['0'..'9']) and (ascii = false) then
    begin
      ascii:= true;
      r:= r + ' ';
    end;
  r := r + s[i];
  end;
  result := trim(AnsiProperCase(r, [' ','''','(',')']));
end;

procedure readPlaylist();
// Read Playlist from musicPlayer and set the following variables.
// numTitlesInPlaylist - number of tracks in playlist
// cRunning, cTitle, cArtist and cGenre - Array of strings of playlist items
// currentSong and currentStatus - Current Song and Status of musicPlayer
// playingIdTag - ID Tag of playing or stopped song in playlist
// playingStatus - Running, Stopped or False
// readSuccess - true if readPlaylist() was successful
var
  Splitted: TStringArray;
  index, i: longint;
const
  cropAt = 27;
begin
  // Get songs from musicPlayer
  execOsaScript(musicPlayer);
  // Get rid of <CR> and <LF> characters, if any
  Status := StringReplace(Status, Chr(13), '', [rfReplaceAll]);
  Status := StringReplace(Status, Chr(10), '', [rfReplaceAll]);
  // Replace all ', ' with ','
  Status := StringReplace(Status, ', ', ',', [rfReplaceAll]);
  // Split comma seperated string Status into TStringArray Splitted
  Splitted := Status.Split(',');
  // First string in Splitted is the number of titles in playlist-
  // If this number is 0, then readPlaylist returned without result.
  numTitlesInPlaylist := StrToInt(Splitted[0]);
  if numTitlesInPlaylist = 0 then
  begin
    readSuccess := False;
    exit;
  end
  else
  begin
    index := 1;
    SetLength(cRunning, numTitlesInPlaylist);
    for i := 1 to numTitlesInPlaylist do
    begin
      cRunning[i - 1] := Splitted[index];
      index := index + 1;
    end;
    SetLength(cTitle, numTitlesInPlaylist);
    for i := 1 to numTitlesInPlaylist do
    begin
      cTitle[i - 1] := seperateNumWithSpace(Splitted[index]);
      index := index + 1;
    end;
    SetLength(cArtist, numTitlesInPlaylist);
    for i := 1 to numTitlesInPlaylist do
    begin
      cArtist[i - 1] := seperateNumWithSpace(Splitted[index]);
      index := index + 1;
    end;
    SetLength(cGenre, numTitlesInPlaylist);
    for i := 1 to numTitlesInPlaylist do
    begin
      cGenre[i - 1] := Splitted[index];
      index := index + 1;
    end;
    // Postprocess the playlist items
    for i := 1 to numTitlesInPlaylist do
    begin
      // Shorten cArtist + cTitle to 2 * cropAt max
      if (length(cArtist[i - 1]) > cropAt) and (length(cTitle[i - 1]) > cropAt) then
      begin
           cTitle[i - 1] := LeftStr(cTitle[i - 1], cropAt);
           cArtist[i - 1] := LeftStr(cArtist[i - 1], cropAt);
      end
      else if length(cArtist[i - 1]) > cropAt then
        cArtist[i - 1] := LeftStr(cArtist[i - 1], cropAt+cropAt-length(cTitle[i - 1]))
      else if length(cTitle[i - 1]) > cropAt then
        cTitle[i - 1] := LeftStr(cTitle[i - 1], cropAt+cropAt-length(cArtist[i - 1]));
      // Set genre to Cortina, if not in Tango, Vals or Milonga
      // and changeToCortina is set to true
      if (changeToCortina = 'true') then
         if ((cGenre[i - 1] <> 'Tango') and (cGenre[i - 1] <> 'Vals')
            and (cGenre[i - 1] <> 'Milonga')) then
                cGenre[i - 1] := 'Cortina';
      // Set currentStatus and currentSong of musicPlayer
      if ((cRunning[i - 1] = 'Playing') or (cRunning[i - 1] = 'Stopped')) then
        begin
          currentStatus := cRunning[i - 1];
          currentSong := cArtist[i - 1] + ' - ' + cTitle[i - 1] + ' - ' + cGenre[i - 1];
        end;
    end;
    playingIdTag := Splitted[index];
    index := index + 1;
    playingStatus := StrToBool(Splitted[index]);
    index := index + 1;
    readSuccess := StrToBool(Splitted[index]);
  end;
end;

procedure writePlaylist(advertising: string);
// Write Playlist to /Sites directory on LocalMac
var
  tfile: TextFile;
  i: longint;
begin
  // If currentStatus=Playing then write playlist to DisplayPlaylistHtml
  if (currentStatus = 'Playing') then
  begin
    if FileExists(displayPlaylistHeader) then
       CopyFile(displayPlaylistHeader, displayPlaylistHtml + '.tmp');
    AssignFile(tfile, displayPlaylistHtml + '.tmp');
    Append(tfile);
     for i := 1 to numTitlesInPlaylist do
     begin
       // Write playlist line
       if (cRunning[i - 1] = 'Playing') then write(tfile, '<p class="TextInColor">')
       else write(tfile, '<p>');
       writeln(tfile, cArtist[i - 1] +
         ' - ' + cTitle[i - 1] + ' - ' + cGenre[i - 1] + '</p>');
     end;
     writeln (tfile, '</div>' + LineEnding + '</body>' + LineEnding + '</html>');
     CloseFile(tfile);
     if FileExists(DisplayPlaylistHtml) then
         DeleteFile(DisplayPlaylistHtml);
     RenameFile(displayPlaylistHtml + '.tmp', displayPlaylistHtml);
  end
  // if currentStatus=Stopped then write advertising + '.htm' to DisplayPlaylistHtml
  else if (currentStatus ='Stopped') then
  begin
    if FileExists(macSitesDir + advertising + '.htm') then
    begin
      if FileExists(DisplayPlaylistHtml) then
         DeleteFile(DisplayPlaylistHtml);
      CopyFile(macSitesDir + advertising + '.htm', displayPlaylistHtml);
    end;
  end;
end;

function playlistStatus(): string;
// Return concatenated string of all songs in a playlist to be able to
// detect future changes in the playlist
var
  i: longint;
  Str: string;
begin
  Str := '';
  if numTitlesInPlaylist = 0 then
    exit('')
  else
  begin
    for i := 1 to numTitlesInPlaylist do
    begin
      Str := Str + cRunning[i - 1] + cTitle[i - 1] + cArtist[i - 1] + cGenre[i - 1];
    end;
    exit(Str);
  end;
end;

procedure readIniFile();
// Read .DisplayPlaylist.ini file
var
  appINI: TIniFile;
const
  DIS = 'DisplayPlaylist';
  POS = 'WindowPosition';
begin
  // Read .INI file $HOME/.DisplayPlaylist.ini
  appINI := TIniFile.Create(GetUserDir() + '.DisplayPlaylist.ini');
  try
    advertising := appINI.ReadString(DIS, 'advertising', 'Advertising');
    musicPlayer := appINI.ReadString(DIS, 'musicPlayer', 'Music');
    macSitesDir := appINI.ReadString(DIS, 'macSitesDir', '');
    browserUrl := appINI.ReadString(DIS, 'browserUrl', '');
    runAsScriptProg := appINI.ReadString(DIS, 'runAsScriptProg', '');
    changeToCortina := appINI.ReadString(DIS, 'changeToCortina', 'true');
    DisplayPlaylistGUI.Top := appINI.ReadInteger(POS, 'Top', 100);
    DisplayPlaylistGUI.Left := appINI.ReadInteger(POS, 'Left', 100);
    DisplayPlaylistGUI.Height := appINI.ReadInteger(POS, 'Height', 363);
    DisplayPlaylistGUI.Width := appINI.ReadInteger(POS, 'Width', 589);
  finally
    appINI.Free;
  end;
end;

procedure writeIniFile();
// Write current Window Position to .DisplayPlaylist.ini file
var
  appINI: TIniFile;
const
  DIS = 'DisplayPlaylist';
  POS = 'WindowPosition';
begin
  // Write .INI file $HOME/.DisplayPlaylist.ini
  appINI := TIniFile.Create(GetUserDir() + '.DisplayPlaylist.ini');
  try;
    appINI.WriteString(DIS, 'advertising', advertising);
    appINI.WriteString(DIS, 'musicPlayer', musicPlayer);
    appINI.WriteInteger(POS, 'Top', DisplayPlaylistGUI.Top);
    appINI.WriteInteger(POS, 'Left', DisplayPlaylistGUI.Left);
    appINI.WriteInteger(POS, 'Height', DisplayPlaylistGUI.Height);
    appINI.WriteInteger(POS, 'Width', DisplayPlaylistGUI.Width);
  finally
    appINI.Free;
  end;
end;

procedure TDisplayPlaylistGUI.ImageElDuendeClick(Sender: TObject);
// Open website when clicking on the logo
begin
  OpenURL('http://www.el-duende.de');
end;

procedure TDisplayPlaylistGUI.ImageElDuendeMouseEnter(Sender: TObject);
// Change cursor to hand when hovering over the logo
begin
  ImageElDuende.Cursor := crHandPoint;
end;

procedure TDisplayPlaylistGUI.CopyrightClick(Sender: TObject);
// Open website when clicking on the copyright text
begin
  OpenURL('http://www.el-duende.de');
end;

procedure TDisplayPlaylistGUI.CopyrightMouseEnter(Sender: TObject);
// Change color to red and cursor to hand when hovering over copyright text
begin
  Copyright.Font.Color := clRed;
  Copyright.Cursor := crHandPoint;
end;

procedure TDisplayPlaylistGUI.CopyrightMouseLeave(Sender: TObject);
// Change color back to blue when cursor leaves the copyright text
begin
  Copyright.Font.Color := clBlue;
end;

procedure TDisplayPlaylistGUI.ComboBoxAdvertisingSelect(Sender: TObject);
// Select Advertising Mode
begin
  if ComboBoxAdvertising.ItemIndex <> -1 then
    advertising := ComboBoxAdvertising.Items.Strings[ComboBoxAdvertising.ItemIndex];
end;

procedure TDisplayPlaylistGUI.ComboBoxMusicPlayerSelect(Sender: TObject);
// Select Music Player
begin
  if ComboBoxMusicPlayer.ItemIndex <> -1 then
    musicPlayer := ComboBoxMusicPlayer.Items.Strings[ComboBoxMusicPlayer.ItemIndex];
end;

procedure TDisplayPlaylistGUI.QuitClick(Sender: TObject);
// Write current window position to $HOME/.DisplayPlaylist.ini
// file and quit program
begin
  // Write current Window Position to .DisplayPlaylist.ini file
  writeIniFile();
  Close;
end;

procedure TDisplayPlaylistGUI.StartBrowserClick(Sender: TObject);
// Start browser to show playlist on Mac monitor
begin
  OpenURL(browserUrl);
end;

procedure TDisplayPlaylistGUI.StartBrowserMouseEnter(Sender: TObject);
// Change color to red and cursor to hand when hovering over copyright text
begin
  StartBrowser.Font.Color := clRed;
  StartBrowser.Cursor := crHandPoint;
end;

procedure TDisplayPlaylistGUI.StartBrowserMouseLeave(Sender: TObject);
// Change color back to blue when cursor leaves the copyright text
begin
  StartBrowser.Font.Color := clBlue;
end;

procedure TDisplayPlaylistGUI.FormCreate(Sender: TObject);
// Start the main DisplayPlaylist program
var
  playerIsPlaying, playerLastPlaying: boolean;
  adSignaledYet, logSignaledYet: boolean;
  currentTitleIdTag, lastTitleIdTag, lastAdvertising: string;
  currentPlaylistTitles, lastPlaylistTitles: string;
begin
  // Read $HOME/.DisplayPlaylist.ini file
  readIniFile();
  // Set path for the AppleScript program - use default if not exists
  // Set LabelAppleScript.Caption to runAsScriptProg
  Status := ExtractFileDir(ParamStr(0));
  Status := StringReplace(Status, 'MacOS', '', [rFReplaceAll]);
  if not FileExists(Status + 'Resources/' + runAsScriptProg) then
    runAsScriptProg := 'runAsScript7Tracks.scpt';
  runAsScript := Status + 'Resources/' + runAsScriptProg;
  LabelAppleScript.Caption := runAsScriptProg;
  // Get Mac user name
  macUser := GetEnvironmentVariable('USER');
  macHostName := getHostName;
  // Set browserURL, if not defined
  if (BrowserUrl = '') then
     BrowserUrl := 'http://' + macHostname + '/~' + macUser;
  // Set macSitesDir, if not defined
  if (macSitesDir = '' ) then
     macSitesDir := GetUserDir + 'Sites/';
  displayPlaylistHtml := macSitesDir + 'DisplayPlaylist.htm';
  displayPlaylistHeader := macSitesDir + 'DisplayPlaylistHeader.htm';
  // Set advertising and musicPlayer in ComboBoxes from .DisplayPlaylist.ini file
  ComboBoxAdvertising.ItemIndex := ComboBoxAdvertising.Items.IndexOf(advertising);
  ComboBoxMusicPlayer.ItemIndex := ComboBoxMusicPlayer.Items.IndexOf(musicPlayer);
  // Start DisplayPlaylist loop
  while (not Application.Terminated) do
  begin
    readPlaylist();
    if readSuccess then
    begin
      LabelStatus.Caption := 'Playlist read succeeded';
      LabelCurrentStatus.Caption := CurrentStatus;
      LabelCurrentSong.Caption := CurrentSong;
      // Set currentPlayingTitles to playlistStatus()
      currentPlaylistTitles := playlistStatus();
      // Set currentTitleIdTag and lastTitleIdTag to the current song
      currentTitleIdTag := playingIdTag;
      lastTitleIdTag := playingIdTag;
      // Set playerIsPlaying and playerLastPlaying to the current player status
      playerIsPlaying := playingStatus;
      playerLastPlaying := playingStatus;
      // Set adSignaledYet and logSignaledYet to false to force a display status update
      adSignaledYet := False;
      logSignaledYet := False;
      lastAdvertising := advertising;
      // Repeat until next title is played
      while (lastTitleIdTag = currentTitleIdTag) do
      begin
        // write playlist, if 'playing' and not signaled yet
        if (playerIsPlaying and (not logSignaledYet)) then
        begin
          // Signal 'Logfile' once, if player is playing;
          writePlaylist('Logfile');
          logSignaledYet := True;
        end;
        // Handel Advertising mode if not playing and not signaled yet
        if ((not playerIsPlaying) and (not adSignaledYet)) then
        begin
          // Signal Advertising once if not "off", if player has stopped
          if (advertising <> 'Off') then
          begin
            writePlaylist(advertising);
          end;
          lastAdvertising := advertising;
          adSignaledYet := True;
        end;
        // Reread playlist from player and check if things have changed
        lastPlaylistTitles := currentPlaylistTitles;
        readPlaylist();
        if readSuccess then
        begin
          LabelStatus.Caption := 'Playlist read succeeded';
          LabelCurrentStatus.Caption := currentStatus;
          LabelCurrentSong.Caption := currentSong;
          currentPlaylistTitles := playlistStatus();
          currentTitleIdTag := playingIdTag;
          playerIsPlaying := playingStatus;
          // Check if playlist has changed
          if (lastPlaylistTitles <> currentPlaylistTitles) then
          begin
            adSignaledYet := False;
            logSignaledYet := False;
          end;
          // Check if player status has changed (playing or stopped)
          if ((playerIsPlaying and (not playerLastPlaying)) or
            ((not playerIsPlaying) and playerLastPlaying)) then
          begin
            playerLastPlaying := playerIsPlaying;
            adSignaledYet := False;
            logSignaledYet := False;
          end;
          // Check if Advertising mode is changed
          if (lastAdvertising <> advertising) then
          begin
            lastAdvertising := advertising;
            logSignaledYet := False;
            adSignaledYet := False;
          end;
        end;
        if (Application.Terminated) then
          exit;
        MyDelay(1000); // Wait 1 second
      end;
    end
    else
    begin
      LabelStatus.Caption := 'Playlist read failed';
    end;
    if (Application.Terminated) then
      exit;
    MyDelay(1000); //Wait 1 second
  end;
end;

initialization

end.
